@@ -1,5 +1,5 @@
 /* Test of <sys/socket.h> substitute.
-   Copyright (C) 2007 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2009 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,8 +28,11 @@ int a[] = { SHUT_RD, SHUT_WR, SHUT_RDWR };
 #endif
 
 int
-main ()
+main (void)
 {
+  struct sockaddr_storage x;
+  sa_family_t i;
+
   /* Check some errno values.  */
   switch (0)
     {
@@ -43,5 +46,8 @@ main ()
       break;
     }
 
+  x.ss_family = 42;
+  i = 42;
+
   return 0;
 }
