@@ -99,15 +99,13 @@ gsasl_step64 (Gsasl_session * sctx, const char *b64input, char **b64output)
 
   res = gsasl_step (sctx, input, input_len, &output, &output_len);
 
-  if (input != NULL)
-    free (input);
+  free (input);
 
   if (res == GSASL_OK || res == GSASL_NEEDS_MORE)
     {
       int tmpres = gsasl_base64_to (output, output_len, b64output, NULL);
 
-      if (output != NULL)
-	free (output);
+      free (output);
 
       if (tmpres != GSASL_OK)
 	return tmpres;
