@@ -30,8 +30,6 @@
 
 #include "utils.h"
 
-#define MAX_LINE_LENGTH BUFSIZ
-
 #define MAXSTEP 50
 #define CLIENT 1
 #define SERVER 0
@@ -124,11 +122,13 @@ static struct sasltv sasltv[] = {
   {SERVER, "SECURID",
    {"YXV0aHppZABhdXRoaWQANDcxMQA=", "", NULL, NULL}, NULL, "authzid",
    "authid", NULL, NULL, NULL, NULL, "4711"},
+#if 0
   {SERVER, "SECURID",
    {"YXV0aHppZABhdXRoaWQANDcxMQA=", "cGlu",
     "YXV0aHppZABhdXRoaWQANDcxMQA0MgA=", "", NULL, NULL}, NULL, "authzid",
    "authid", NULL, NULL, NULL, NULL, "4711", NULL, "42",
    GSASL_SECURID_SERVER_NEED_NEW_PIN},
+#endif
   {SERVER, "SECURID",
    {"YXV0aHppZABhdXRoaWQANDcxMQA=", "cGluMTc=",
     "YXV0aHppZABhdXRoaWQANDcxMQAyMwA=", "", NULL, NULL}, NULL, "authzid",
@@ -142,261 +142,134 @@ static struct sasltv sasltv[] = {
 };
 
 static int
-client_callback_authorization_id (Gsasl_session_ctx * xctx,
-				  char *out, size_t * outlen)
+cb (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 {
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = sasltv[i].authzid ? strlen (sasltv[i].authzid) : 0;
+  int rc = GSASL_NO_CALLBACK;
+  int i = 0, j = 0;
 
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
+  if (gsasl_callback_hook_get (ctx))
+    i = *(int *) gsasl_callback_hook_get (ctx);
+  if (gsasl_session_hook_get (sctx))
+    j = *(int *) gsasl_session_hook_get (sctx);
 
-  *outlen = needlen;
-  if (out && sasltv[i].authzid)
-    memcpy (out, sasltv[i].authzid, needlen);
+  if (j < 0 || j > 5)
+    fail ("j out of bounds: %d\n", j);
 
-  return GSASL_OK;
-}
-
-static int
-client_callback_authentication_id (Gsasl_session_ctx * xctx,
-				   char *out, size_t * outlen)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].authid);
-
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *outlen = needlen;
-  if (out)
-    memcpy (out, sasltv[i].authid, needlen);
-
-  return GSASL_OK;
-}
-
-static int
-client_callback_password (Gsasl_session_ctx * xctx, char *out,
-			  size_t * outlen)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].password);
-
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *outlen = needlen;
-  if (out)
-    memcpy (out, sasltv[i].password, needlen);
-
-  return GSASL_OK;
-}
-
-static int
-server_callback_validate (Gsasl_session_ctx * xctx,
-			  const char *authorization_id,
-			  const char *authentication_id, const char *password)
-{
-  Gsasl_ctx *ctx = gsasl_server_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-
-  if (authorization_id && sasltv[i].authzid &&
-      strcmp (authorization_id, sasltv[i].authzid) != 0)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if ((authorization_id == NULL && sasltv[i].authzid != NULL) ||
-      (authorization_id != NULL && sasltv[i].authzid == NULL))
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (authentication_id && sasltv[i].authid &&
-      strcmp (authentication_id, sasltv[i].authid) != 0)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (strcmp (password, sasltv[i].password) != 0)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  return GSASL_OK;
-}
-
-static int
-server_callback_retrieve (Gsasl_session_ctx * xctx,
-			  const char *authentication_id,
-			  const char *authorization_id,
-			  const char *realm, char *key, size_t * keylen)
-{
-  Gsasl_ctx *ctx = gsasl_server_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].password);
-
-  if (*keylen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *keylen = strlen (sasltv[i].password);
-  if (key)
-    memcpy (key, sasltv[i].password, needlen);
-
-  return GSASL_OK;
-}
-
-static int
-client_callback_service (Gsasl_session_ctx * ctx,
-			 char *srv,
-			 size_t * srvlen,
-			 char *host,
-			 size_t * hostlen, char *srvname, size_t * srvnamelen)
-{
-  if (srvlen)
-    *srvlen = 0;
-  if (hostlen)
-    *hostlen = 0;
-  if (srvnamelen)
-    *srvnamelen = 0;
-
-  return GSASL_OK;
-}
-
-static int
-client_callback_anonymous (Gsasl_session_ctx * xctx, char *out,
-			   size_t * outlen)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].anonymous);
-
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *outlen = needlen;
-  if (out)
-    memcpy (out, sasltv[i].anonymous, strlen (sasltv[i].anonymous));
-
-  return GSASL_OK;
-}
-
-static int
-server_callback_anonymous (Gsasl_session_ctx * xctx, const char *token)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-
-  return strcmp (sasltv[i].anonymous, token) == 0 ? GSASL_OK :
-    GSASL_AUTHENTICATION_ERROR;
-}
-
-static int
-server_callback_external (Gsasl_session_ctx * xctx)
-{
-  return GSASL_OK;
-}
-
-static int
-client_callback_passcode (Gsasl_session_ctx * xctx, char *out,
-			  size_t * outlen)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].passcode);
-
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *outlen = needlen;
-  if (out)
-    memcpy (out, sasltv[i].passcode, needlen);
-
-  return GSASL_OK;
-}
-
-static int
-client_callback_pin (Gsasl_session_ctx * xctx, char *suggestion,
-		     char *out, size_t * outlen)
-{
-  Gsasl_ctx *ctx = gsasl_client_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  size_t needlen = strlen (sasltv[i].pin);
-
-  if (suggestion && sasltv[i].suggestpin &&
-      strcmp (suggestion, sasltv[i].suggestpin) != 0)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if ((suggestion == NULL && sasltv[i].suggestpin != NULL) ||
-      (suggestion != NULL && sasltv[i].suggestpin == NULL))
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (*outlen < needlen)
-    return GSASL_TOO_SMALL_BUFFER;
-
-  *outlen = needlen;
-  if (out)
-    memcpy (out, sasltv[i].pin, needlen);
-
-  return GSASL_OK;
-}
-
-static int
-server_callback_securid (Gsasl_session_ctx * xctx,
-			 const char *authentication_id,
-			 const char *authorization_id,
-			 const char *passcode,
-			 char *pin, char *suggestpin, size_t * suggestpinlen)
-{
-  Gsasl_ctx *ctx = gsasl_server_ctx_get (xctx);
-  int i = *(int *) gsasl_application_data_get (ctx);
-  int res;
-
-  if (strcmp (passcode, sasltv[i].passcode) != 0)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (sasltv[i].securidrc == GSASL_SECURID_SERVER_NEED_NEW_PIN)
+  switch (prop)
     {
-      res = sasltv[i].securidrc;
-      sasltv[i].securidrc = GSASL_OK;
+    case GSASL_AUTHID:
+      gsasl_property_set (sctx, prop, sasltv[i].authid);
+      rc = GSASL_OK;
+      break;
 
-      if (sasltv[i].suggestpin)
-	{
-	  if (*suggestpinlen)
-	    *suggestpinlen = strlen (sasltv[i].suggestpin);
-	  if (suggestpin)
-	    memcpy (suggestpin, sasltv[i].suggestpin,
-		    strlen (sasltv[i].suggestpin));
-	}
-      else if (*suggestpinlen)
-	*suggestpinlen = 0;
-    }
-  else if (sasltv[i].securidrc ==
-	   GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE)
-    {
-      res = sasltv[i].securidrc;
-      sasltv[i].securidrc = GSASL_OK;
-    }
-  else
-    {
-      res = sasltv[i].securidrc;
+    case GSASL_AUTHZID:
+      gsasl_property_set (sctx, prop, sasltv[i].authzid);
+      rc = GSASL_OK;
+      break;
 
-      if (pin && sasltv[i].pin && strcmp (pin, sasltv[i].pin) != 0)
-	return GSASL_AUTHENTICATION_ERROR;
+    case GSASL_PASSWORD:
+      gsasl_property_set (sctx, prop, sasltv[i].password);
+      rc = GSASL_OK;
+      break;
 
-      if ((pin == NULL && sasltv[i].pin != NULL) ||
-	  (pin != NULL && sasltv[i].pin == NULL))
-	return GSASL_AUTHENTICATION_ERROR;
+    case GSASL_ANONYMOUS_TOKEN:
+      gsasl_property_set (sctx, prop, sasltv[i].anonymous);
+      rc = GSASL_OK;
+      break;
 
-      if (*suggestpinlen)
-	*suggestpinlen = 0;
+    case GSASL_SERVICE:
+      rc = GSASL_OK;
+      break;
+
+    case GSASL_PASSCODE:
+      gsasl_property_set (sctx, prop, sasltv[i].passcode);
+      rc = GSASL_OK;
+      break;
+
+    case GSASL_SUGGESTED_PIN:
+    case GSASL_PIN:
+      {
+	const char *suggestion = gsasl_property_fast (sctx, GSASL_SUGGESTED_PIN);
+	if (suggestion && sasltv[i].suggestpin
+	    && strcmp (suggestion, sasltv[i].suggestpin) != 0)
+	  return GSASL_AUTHENTICATION_ERROR;
+
+	if ((suggestion == NULL && sasltv[i].suggestpin != NULL) ||
+	    (suggestion != NULL && sasltv[i].suggestpin == NULL))
+	  return GSASL_AUTHENTICATION_ERROR;
+
+	gsasl_property_set (sctx, prop, sasltv[i].pin);
+	rc = GSASL_OK;
+      }
+
+    case GSASL_REALM:
+      break;
+
+    case GSASL_VALIDATE_EXTERNAL:
+      rc = GSASL_OK;
+      break;
+
+    case GSASL_VALIDATE_ANONYMOUS:
+      if (strcmp (sasltv[i].anonymous,
+		  gsasl_property_fast (sctx, GSASL_ANONYMOUS_TOKEN)) == 0)
+	rc = GSASL_OK;
+      else
+	rc = GSASL_AUTHENTICATION_ERROR;
+      break;
+
+    case GSASL_VALIDATE_SECURID:
+      {
+	const char *passcode = gsasl_property_fast (sctx, GSASL_PASSCODE);
+	const char *pin = gsasl_property_fast (sctx, GSASL_PIN);
+
+	if (strcmp (passcode, sasltv[i].passcode) != 0)
+	  return GSASL_AUTHENTICATION_ERROR;
+
+	if (sasltv[i].securidrc == GSASL_SECURID_SERVER_NEED_NEW_PIN)
+	  {
+	    rc = sasltv[i].securidrc;
+	    sasltv[i].securidrc = GSASL_OK;
+
+	    if (sasltv[i].suggestpin)
+	      {
+		gsasl_property_set (sctx, GSASL_SUGGESTED_PIN,
+				    sasltv[i].suggestpin);
+	      }
+	  }
+	else if (sasltv[i].securidrc ==
+		 GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE)
+	  {
+	    rc = sasltv[i].securidrc;
+	    sasltv[i].securidrc = GSASL_OK;
+	  }
+	else
+	  {
+	    rc = sasltv[i].securidrc;
+
+	    if (pin && sasltv[i].pin && strcmp (pin, sasltv[i].pin) != 0)
+	      return GSASL_AUTHENTICATION_ERROR;
+
+	    if ((pin == NULL && sasltv[i].pin != NULL) ||
+		(pin != NULL && sasltv[i].pin == NULL))
+	      return GSASL_AUTHENTICATION_ERROR;
+	  }
+      }
+      break;
+
+    default:
+      printf ("Unknown property %d\n", prop);
+      break;
     }
 
-  return res;
+  return rc;
 }
 
 void
 doit (void)
 {
-  Gsasl_ctx *ctx = NULL;
-  Gsasl_session_ctx *xctx = NULL;
-  char output[MAX_LINE_LENGTH];
-  size_t outputlen;
+  Gsasl *ctx = NULL;
+  Gsasl_session *sctx = NULL;
+  char *out = NULL;
   size_t i, j;
   int res;
 
@@ -413,36 +286,25 @@ doit (void)
       return;
     }
 
-  gsasl_client_callback_authentication_id_set
-    (ctx, client_callback_authentication_id);
-  gsasl_client_callback_authorization_id_set
-    (ctx, client_callback_authorization_id);
-  gsasl_client_callback_password_set (ctx, client_callback_password);
-  gsasl_server_callback_validate_set (ctx, server_callback_validate);
-  gsasl_server_callback_retrieve_set (ctx, server_callback_retrieve);
-  gsasl_client_callback_service_set (ctx, client_callback_service);
-  gsasl_client_callback_anonymous_set (ctx, client_callback_anonymous);
-  gsasl_server_callback_anonymous_set (ctx, server_callback_anonymous);
-  gsasl_server_callback_external_set (ctx, server_callback_external);
-  gsasl_client_callback_passcode_set (ctx, client_callback_passcode);
-  gsasl_client_callback_pin_set (ctx, client_callback_pin);
-  gsasl_server_callback_securid_set (ctx, server_callback_securid);
+  gsasl_callback_set (ctx, cb);
 
-  outputlen = sizeof (output);
-  res = gsasl_client_listmech (ctx, output, &outputlen);
+  res = gsasl_client_mechlist (ctx, &out);
   if (res != GSASL_OK)
-    fail ("gsasl_client_listmech() failed (%d):\n%s\n",
+    fail ("gsasl_client_mechlist() failed (%d):\n%s\n",
 	  res, gsasl_strerror (res));
+  success("client_mechlist: %s\n", out);
+  gsasl_free (out); out = NULL;
 
-  outputlen = sizeof (output);
-  res = gsasl_server_listmech (ctx, output, &outputlen);
+  res = gsasl_server_mechlist (ctx, &out);
   if (res != GSASL_OK)
-    fail ("gsasl_server_listmech() failed (%d):\n%s\n",
+    fail ("gsasl_server_mechlist() failed (%d):\n%s\n",
 	  res, gsasl_strerror (res));
+  success("server_mechlist: %s\n", out);
+  gsasl_free (out); out = NULL;
 
   for (i = 0; i < sizeof (sasltv) / sizeof (sasltv[0]); i++)
     {
-      gsasl_application_data_set (ctx, &i);
+      gsasl_callback_hook_set (ctx, &i);
 
       if (debug)
 	printf ("Entry %d %s mechanism %s:\n",
@@ -456,9 +318,9 @@ doit (void)
 	continue;
 
       if (sasltv[i].clientp)
-	res = gsasl_client_start (ctx, sasltv[i].mech, &xctx);
+	res = gsasl_client_start (ctx, sasltv[i].mech, &sctx);
       else
-	res = gsasl_server_start (ctx, sasltv[i].mech, &xctx);
+	res = gsasl_server_start (ctx, sasltv[i].mech, &sctx);
       if (res != GSASL_OK)
 	{
 	  fail ("SASL %s start for mechanism %s failed (%d):\n%s\n",
@@ -469,31 +331,24 @@ doit (void)
 
       for (j = 0; sasltv[i].step[j]; j += 2)
 	{
-	  if (sasltv[i].clientp)
-	    gsasl_client_application_data_set (xctx, &j);
-	  else
-	    gsasl_server_application_data_set (xctx, &j);
+	  gsasl_session_hook_set (sctx, &j);
 
 	  if (debug)
 	    printf ("Input : %s\n",
 		    sasltv[i].step[j] ? sasltv[i].step[j] : "");
 
-	  output[0] = '\0';
-	  outputlen = sizeof (output);
-	  if (sasltv[i].clientp)
-	    res = gsasl_client_step_base64 (xctx, sasltv[i].step[j],
-					    output, outputlen);
-	  else
-	    res = gsasl_server_step_base64 (xctx, sasltv[i].step[j],
-					    output, outputlen);
+	  res = gsasl_step64 (sctx, sasltv[i].step[j], &out);
 
 	  if (debug)
-	    printf ("Output: %s\n", output);
+	    printf ("Output: %s\n", out ? out : "(null)");
 
 	  if (res != GSASL_OK && res != GSASL_NEEDS_MORE)
-	    break;
+	    {
+	      fail ("gsasl_step64 failed (%d): %s", res, gsasl_strerror (res));
+	      break;
+	    }
 
-	  if (strlen (output) !=
+	  if (strlen (out) !=
 	      strlen (sasltv[i].step[j + 1] ? sasltv[i].step[j + 1] : ""))
 	    {
 	      printf ("Expected: %s\n", sasltv[i].step[j + 1] ?
@@ -505,7 +360,7 @@ doit (void)
 	      break;
 	    }
 
-	  if (strcmp (output, sasltv[i].step[j + 1] ?
+	  if (strcmp (out, sasltv[i].step[j + 1] ?
 		      sasltv[i].step[j + 1] : "") != 0)
 	    {
 	      printf ("Expected: %s\n", sasltv[i].step[j + 1] ?
@@ -516,6 +371,8 @@ doit (void)
 	      break;
 	    }
 
+	  gsasl_free (out); out = NULL;
+
 	  if (strcmp (sasltv[i].mech, "SECURID") != 0 && res == GSASL_OK)
 	    break;
 	}
@@ -533,10 +390,25 @@ doit (void)
 	printf ("PASS: simple %s %s %d\n", sasltv[i].mech,
 		sasltv[i].clientp ? "client" : "server", i);
 
-      if (sasltv[i].clientp)
-	gsasl_client_finish (xctx);
-      else
-	gsasl_server_finish (xctx);
+      {
+	size_t outlen;
+
+	res = gsasl_encode (sctx, "foo", 3, &out, &outlen);
+	if (res != GSASL_OK)
+	  fail ("gsasl_encode %d: %s\n", res, gsasl_strerror (res));
+	if (outlen != 3 && memcmp (out, "foo", outlen) != 0)
+	  fail ("gsasl_encode memcmp: %.*s\n", outlen, out);
+	gsasl_free (out); out = NULL;
+
+	res = gsasl_decode (sctx, "foo", 3, &out, &outlen);
+	if (res != GSASL_OK)
+	  fail ("gsasl_decode %d: %s\n", res, gsasl_strerror (res));
+	if (outlen != 3 && memcmp (out, "foo", outlen) != 0)
+	  fail ("gsasl_decode memcmp: %.*s\n", outlen, out);
+	gsasl_free (out); out = NULL;
+      }
+
+      gsasl_finish (sctx);
 
       if (debug)
 	printf ("\n");
